
package com.laszlomarket;

import org.hibernate.HibernateException;
import org.hibernate.MappingException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Singleton class that creates and returns an open Hibernate <code>Session</code> to the user.
 * 
 * 
 * @author Norman Klein
 *
 */
/* 
 * This class was created to encapsulate the Hibernate SessionFactory and assist
 * the service layer
 * 
 */ 
public class ConnectionFactory {

	private static Log log = LogFactory.getLog(ConnectionFactory.class);
	private static ConnectionFactory instance = null;
	private SessionFactory sessionFactory = null;

       /**
    	* The Hibernate Configuration contains all the classes to be
    	* persisted by Hibernate.  For each class persisted, Hibernate will
    	* expect to find a ClassName.hbm.xml file in the same location as the
    	* class file.  This XML file will define the mapping between the Java
    	* object and the database.
    	* 
    	* To add additional classes to the configuration
    	* <pre> 
    	* Configuration cfg = new Configuration().
    	*                     addClass(Foo.class).
    	*                     addClass(Bar.class);
        * </pre> 
    	*/
	private ConnectionFactory() {

		// Establish SessionFactory for Hibernate
		try {
			Configuration cfg = new Configuration().addClass(Product.class);
			sessionFactory = cfg.buildSessionFactory();
		}
		catch (MappingException e) {
			// Throw an unchecked RuntimeExcpetion to cause the user's request to fail
			log.warn("Mapping Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
		catch (HibernateException e) {
			// Throw an unchecked RuntimeExcpetion to cause the user's request to fail
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}

	}

	/** 
	 * getInstance() returns the instance of the ConnectionFactory singleton.
	 * 
	 * Example call to retrieve session:
	 * 
	 * <pre>
	 * Session session = ConnectionFactory.getInstance().getSession();
	 * </pre>
	 * 
	 * @return Instance of the <code>ConnectionFactory</code> singleton.
	 */
	public static synchronized ConnectionFactory getInstance() {

		 // If the instance of the Singleton has not been created, create and return
		if (instance == null) {
			instance = new ConnectionFactory();
		}
		return instance;
	}

	/**
	 * getSession() returns a Hibernate <code>Session</code>
	 * 
	 * @return <code>Session</code> retrieved from Hibernate <Code>SessionFactory</code>
	 */
	public Session getSession() {

		try {
			// Hibernate Session Factory returns an open session to the caller.
			Session s = sessionFactory.openSession();
			return s;
		}
		catch (HibernateException e) {
			// Throw an unchecked RuntimeExcpetion to cause the user's request to fail
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
	}

}

