package com.laszlomarket;

import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.ObjectNotFoundException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Hibernate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Provides an interface used to access <code>Product</code> objects from the 
 * database using Hibernate.
 * 
 * @author Norman Klein
 *
 */

/*
 * The ProductDAO class uses the Singleton design pattern to provide an interface 
 * to the Struts application allowing it to work with Product objects through 
 * the persistence layer.
 * 
 * All Hibernate-related code exists at this layer.  
 *  
 */
public class ProductDAO {

	private static Log log = LogFactory.getLog(ProductDAO.class);
	private static ProductDAO instance = null;
	private ProductDAO() { }

	/**
	 * getInstance() returns the instance of the <code>ProductDAO</code> singleton.
	 * 
	 * @return <code>ProductDAO</code> singleton.
	 */
	public static synchronized ProductDAO getInstance() {
	
		// Creates a Singleton instance
		if (instance == null) {
			instance = new ProductDAO();
		}
		return instance;
	}

	/**
	 * getProduct() returns <code>Product</code> object from database with given id.  If
	 * the product is not found, then null is returned.
	 * 
	 * @param <code>product</code> contained an <code>id</code>
	 * @return <code>Product</code> with given id, if it exists.  Otherwise, returns null.
	 */
	public List getProduct(Product product) {

	    // Use the ConnectionFactory to retrieve an open Hibernate Session.
		Session session = ConnectionFactory.getInstance().getSession();

		try {
			// Build HQL (Hibernate Query Language) query to retrieve a list
			// of all the products currently stored by Hibernate.
			Query query = session.createQuery(
					"from Product product where product.id = :id");
			query.setParameter("id", product.getId(), Hibernate.LONG);
log.warn("query.list=" + query.list());
			return query.list();
		}
		
		// No Product exists with the requested ID, just return null rather 
		// than throwing an Exception.
		catch (ObjectNotFoundException onfe) {
            log.warn("no object found with the requested id, " + product.getId());
			return null;
		}
		catch (HibernateException e) {

			// Convert Hibernate exceptions into unchecked RuntimeExceptions. So that 
			// the user's request returns an error.
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
		 
        // Always close the Hibernate session when finished
		finally {
			if (session != null) {
				try {
					session.close();
				}
				catch (HibernateException e) {
					log.warn("Hibernate Exception" + e.getMessage());
					throw new RuntimeException(e);
				}
			}
		}
	}

	/** 
	 * updateProduct() updates specfied <code>Product</code> through Hibernate.
     *
	 * @param product A new <code>Product</code> to be added.
	 */
	public void updateProduct(Product product) {
		
		// Retrieve an open Hibernate Session.
		Session session = ConnectionFactory.getInstance().getSession();

		try {
			// Update the state of the product using the Hibernate session's update method
			// Call the flush method to ensure that the object is saved
			session.update(product);
			session.flush();
		}
		catch (HibernateException e) {
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
      
        // Always close the Hibernate session when finished
		finally {
			if (session != null) {
				try {
					session.close();
				}
				catch (HibernateException e) {
					log.warn("Hibernate Exception" + e.getMessage());
					throw new RuntimeException(e);
				}
			}
		}
	}

	/**
	 * getProducts() returns list of all matching <code>Product</code> objects stored in the database. 
	 * 
	 * @return <code>List</code> of <code>Product</code> objects.
	 */
	public List getProducts(Product product) {

		// Use the ConnectionFactory to retrieve an open
		// Hibernate Session.
		Session session = ConnectionFactory.getInstance().getSession();

		try {
			// Build HQL (Hibernate Query Language) query to retrieve a list
			// of all the products currently stored by Hibernate.
			Query query = session.createQuery(
					"from Product product order by product.title");
log.warn("XX2 list=" + query.list());
			return query.list();

		}
		catch (HibernateException e) {
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}

		// Close the Hibernate session
		finally {
			if (session != null) {
				try {
					session.close();
				}
				catch (HibernateException e) {
					log.warn("Hibernate Exception" + e.getMessage());
					throw new RuntimeException(e);
				}
			}
		}
	}

	/**
	 * addProduct() inserts new <code>Product</code> into the database through Hibernate.
	 * 
	 * @param product A new <code>Product</code> to be added.
	 */
	public void addProduct(Product product) {

		Session session = ConnectionFactory.getInstance().getSession();

		try {
			session.save(product);
			session.flush();
		}
		catch (HibernateException e) {
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
		 
		// Close the Hibernate session
		finally {
			if (session != null) {
				try {
					session.close();
				}
				catch (HibernateException e) {
					log.warn("Hibernate Exception" + e.getMessage());
					throw new RuntimeException(e);
				}
			}
		}
	}


	/**
	 * deleteProduct() delete the specified <code>Product</code> from the database through Hibernate.
	 * 
	 * @param id The <code>Long</code> id of desired <code>Product</code> 
	 */
	public void deleteProduct(Product product) {

		Session session = ConnectionFactory.getInstance().getSession();

		try {
			product = (Product) session.get(Product.class, product.getId());
			session.delete(product);
			session.flush();
		}
		catch (HibernateException e) {
			log.warn("Hibernate Exception" + e.getMessage());
			throw new RuntimeException(e);
		}
		 
		// Close the Hibernate session
		finally {
			if (session != null) {
				try {
					session.close();
				}
				catch (HibernateException e) {
					log.warn("Hibernate Exception" + e.getMessage());
					throw new RuntimeException(e);
				}
			}
		}
	}
}

